package ru.dbotthepony.kstarbound.io

import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kvector.vector.Color

object ColorTypeAdapter : TypeAdapter<Color>() {
	override fun write(out: JsonWriter, value: Color) {
		TODO("Not yet implemented")
	}

	override fun read(`in`: JsonReader): Color {
		when (val type = `in`.peek()) {
			JsonToken.BEGIN_ARRAY -> {
				`in`.beginArray()
				val red = `in`.nextDouble()
				val green = `in`.nextDouble()
				val blue = `in`.nextDouble()

				if (red % 1.0 == 0.0 && green % 1.0 == 0.0 && blue % 1.0 == 0.0) {
					val alpha = `in`.peek().let { if (it == JsonToken.END_ARRAY) 255.0 else `in`.nextDouble() }
					`in`.endArray()

					return Color(
						red.toFloat() / 255f,
						green.toFloat() / 255f,
						blue.toFloat() / 255f,
						alpha.toFloat() / 255f,
					)
				} else {
					val alpha = `in`.peek().let { if (it == JsonToken.END_ARRAY) 1.0 else `in`.nextDouble() }
					`in`.endArray()

					return Color(
						red.toFloat(),
						green.toFloat(),
						blue.toFloat(),
						alpha.toFloat(),
					)
				}
			}

			JsonToken.BEGIN_OBJECT -> {
				`in`.beginObject()

				val keyed = mutableMapOf<String, Double>()

				while (`in`.peek() != JsonToken.END_OBJECT) {
					keyed[`in`.nextName()] = `in`.nextDouble()
				}

				if (keyed.isEmpty())
					throw IllegalArgumentException("Object is empty")

				var values = 0

				val red = keyed["red"]?.also { values++ } ?: keyed["r"]?.also { values++ } ?: 255.0
				val green = keyed["green"]?.also { values++ } ?: keyed["g"]?.also { values++ } ?: 255.0
				val blue = keyed["blue"]?.also { values++ } ?: keyed["b"]?.also { values++ } ?: 255.0
				val alpha = keyed["alpha"]?.also { values++ } ?: keyed["a"]?.also { values++ } ?: 255.0
				`in`.endObject()

				if (values == 0) {
					throw IllegalArgumentException("Object is not a color")
				}

				if (red % 1.0 == 0.0 && green % 1.0 == 0.0 && blue % 1.0 == 0.0 && alpha % 1.0 == 0.0) {
					return Color(
						red.toFloat() / 255f,
						green.toFloat() / 255f,
						blue.toFloat() / 255f,
						alpha.toFloat() / 255f,
					)
				} else {
					return Color(
						red.toFloat(),
						green.toFloat(),
						blue.toFloat(),
						alpha.toFloat(),
					)
				}
			}

			JsonToken.NUMBER -> return Color(`in`.nextInt())
			JsonToken.STRING -> {
				val name = `in`.nextString()
				return Color.PRE_DEFINED_MAP[name] ?: throw IllegalArgumentException("Unknown pre defined color name $name")
			}

			else -> throw IllegalArgumentException("Expected array, object or number; got $type")
		}
	}
}
